@@ -21,8 +21,8 @@ from m3_gar.models import (
     Rooms,
 )
 from m3_rest_gar.util import (
-    filter_by_parentobjid,
-    filter_by_name_with_parents,
+    get_hierarchy_models,
+    is_objectguid,
 )
 
 
@@ -34,42 +34,80 @@ class CharInFilter(BaseInFilter, CharFilter):
     pass
 
 
-class HierarchyParentFilter(CharFilter):
+class HierarchyFilter(CharFilter):
+    """
+    Фильтр в иерархии адресных объектов.
+    На входе ожидается строка вида `hierarchy:filter_str`,
+    где hierarchy - название иерархии, filter_str - значение,
+    по которому нужно фильтровать. Класс-наследник определяет
+    поведение фильтра для filter_str.
+    """
+
+    def filter(self, qs, value):
+        if value:
+            try:
+                hierarchy, filter_str = value.split(':')
+            except ValueError:
+                pass
+            else:
+                filter_expression = Q()
+
+                for hierarchy_model in get_hierarchy_models(hierarchy):
+                    filter_for_hierarchy = self.get_filter_for_hierarchy(hierarchy_model, filter_str)
+                    if filter_for_hierarchy:
+                        filter_expression |= filter_for_hierarchy
+
+                qs = qs.filter(filter_expression)
+
+        return qs
+
+    def get_filter_for_hierarchy(self, hierarchy_model, filter_value):
+        """Возвращает фильтр по переданному значению для модели иерархии.
+
+        Args:
+            hierarchy_model: Класс модели иерархии
+            filter_value: Строковое значение, по которому нужно фильтровать
+
+        Returns:
+            Выражение, содержащее необходимые фильтры
+        """
+
+        return Q()
+
+
+class HierarchyParentFilter(HierarchyFilter):
     """
     Фильтр по objectid родителя в иерархии адресных объектов.
     На вход фильтра ожидается строка вида `hierarchy:parentobjid`
-    Более подробно см. `m3_rest_gar.util.filter_by_parentobjid`
     """
 
-    def filter(self, qs, value):
-        if value:
-            try:
-                hierarchy, objectid = value.split(':')
-            except ValueError:
-                pass
-            else:
-                qs = filter_by_parentobjid(hierarchy, qs, objectid)
+    def get_filter_for_hierarchy(self, hierarchy_model, filter_value):
+        if is_objectguid(filter_value):
+            filter_key = 'parentobjid__objectguid'
+        else:
+            filter_key = 'parentobjid__objectid'
 
-        return qs
+        return Q(
+            objectid__in=hierarchy_model.objects.filter(**{
+                'isactive': True,
+                filter_key: filter_value,
+            }).values('objectid'),
+        )
 
 
-class HierarchyNameWithParentsFilter(CharFilter):
+class HierarchyNameWithParentsFilter(HierarchyFilter):
     """
     Фильтр по полю name_with_parents в иерархии адресных объектов.
     На вход фильтра ожидается строка вида `hierarchy:str_value`
-    Более подробно см. `m3_rest_gar.util.filter_by_name_with_parents`
     """
 
-    def filter(self, qs, value):
-        if value:
-            try:
-                hierarchy, srt_value = value.split(':')
-            except ValueError:
-                pass
-            else:
-                qs = filter_by_name_with_parents(hierarchy, qs, srt_value)
-
-        return qs
+    def get_filter_for_hierarchy(self, hierarchy_model, filter_value):
+        return Q(
+            objectid__in=hierarchy_model.objects.filter(**{
+                'isactive': True,
+                'name_with_parents__icontains': filter_value,
+            }).values('objectid'),
+        )
 
 
 class AddrObjFilter(FilterSet):
@@ -81,12 +119,13 @@ class AddrObjFilter(FilterSet):
     region_code = NumberInFilter(field_name='region_code')
     name = CharFilter(lookup_expr='icontains')
     name__exact = CharFilter(lookup_expr='exact')
+    name_with_typename = CharFilter(lookup_expr='icontains')
     typename = CharInFilter(field_name='typename')
     name_with_parents = HierarchyNameWithParentsFilter()
 
     class Meta:
         model = AddrObj
-        fields = ['level', 'parent', 'name', 'name__exact', 'typename']
+        fields = ['level', 'parent', 'name', 'name__exact', 'name_with_typename', 'typename']
 
 
 class HousesFilter(FilterSet):
